package fr.natan.addressmicroservice.t_controllers;

import fr.natan.addressmicroservice.entity.Address;
import fr.natan.addressmicroservice.exceptions.AddressAlreadyExistsException;
import fr.natan.addressmicroservice.exceptions.AddressOneOrMoreFieldsEmptyException;
import fr.natan.addressmicroservice.exceptions.AddressNotFoundException;
import fr.natan.addressmicroservice.service.service.AddressService;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
public class AddressController {
    private final AddressService addressService;

    public AddressController(AddressService addressService) {
        this.addressService = addressService;
    }

    @RequestMapping(value = "/addresses", method = RequestMethod.POST)
    private Address createAddress(@RequestBody Address address) throws AddressOneOrMoreFieldsEmptyException,
            AddressAlreadyExistsException {
        return addressService.createAddress(address);
    }

    @RequestMapping(value = "/addresses/{addressID}", method = RequestMethod.DELETE)
    private void deleteAddress(@PathVariable(name = "addressID") Long addressID) throws AddressNotFoundException {
        addressService.deleteAddress(addressID);
    }

    @RequestMapping(value = "/addresses", method = RequestMethod.GET)
    private List<Address> getAllAddresses(){
        return addressService.getAllAddresses();
    }

    @RequestMapping(value = "/addresses/{addressID}", method = RequestMethod.GET)
    private Address getAddress(@PathVariable(name = "addressID") Long addressID) throws AddressNotFoundException {
        return addressService.getAddressByID(addressID);
    }

    @RequestMapping(value = "/addresses/{addressID}")
    private Address updateAddress(@PathVariable(name = "addressID") Long addressID, @RequestBody Address address) throws AddressNotFoundException {
        return addressService.updateAddress(addressID, address);
    }
}
