package fr.natan.companymicroservice.service.service;

import fr.natan.companymicroservice.exceptions.CompanyNameEmptyException;
import fr.natan.companymicroservice.exceptions.CompanyNotFoundException;
import fr.natan.companymicroservice.model.entity.Company;
import fr.natan.companymicroservice.exceptions.CompanyNameAlreadyExistsException;
import fr.natan.companymicroservice.repository.CompanyRepository;
import fr.natan.companymicroservice.service.contract.CompanyContract;
import org.springframework.stereotype.Service;

import java.sql.Date;
import java.sql.Time;
import java.util.List;

@Service
public class CompanyService implements CompanyContract {

    private final CompanyRepository companyRepository;

    public CompanyService(CompanyRepository companyRepository) {
        this.companyRepository = companyRepository;
    }

    private Company normalize(Company company){
        company.setCompanyName(company.getCompanyName().strip());
        return company;
    }

    private boolean isValidCompany(Company company){
        if(company.getCompanyName().isEmpty()){
            return false;
        }
        else{
            return true;
        }
    }
    @Override
    public Company getCompanyByName(String companyName) {
        return companyRepository.findByCompanyName(companyName);
    }

    @Override
    public Company createCompany(Company company) throws CompanyNameAlreadyExistsException, CompanyNameEmptyException {
        company = normalize(company);
        if(isValidCompany(company)==false){
            throw new CompanyNameEmptyException("company name is empty");
        }

        Company savedCompany = getCompanyByName(company.getCompanyName());
        if(savedCompany!=null){
            throw new CompanyNameAlreadyExistsException("company already exist");
        }
        company.setCreatedDate(new Date(System.currentTimeMillis())+": "+new Time(System.currentTimeMillis()));
        return companyRepository.save(company);
    }

    @Override
    public List<Company> getAllCompanies() {
        return companyRepository.findAll();
    }

    @Override
    public Company getCompanyByID(Long companyID) throws CompanyNotFoundException {
        return companyRepository.findById(companyID).orElseThrow(
                ()-> new CompanyNotFoundException("company not found")
        );
    }

    @Override
    public Company updateCompany(Long companyID,Company company) throws CompanyNotFoundException {
        getCompanyByID(companyID);//verify if company to update exists, if not raise exception
        return companyRepository.save(company);
    }

    @Override
    public void deleteCompany(Long companyID) throws CompanyNotFoundException {
        Company company = getCompanyByID(companyID);//verify if company to delete exists, if not raise exception
        companyRepository.delete(company);
    }
}
