package fr.natan.companymicroservice.t_controller;

import fr.natan.companymicroservice.exceptions.CompanyNameAlreadyExistsException;
import fr.natan.companymicroservice.exceptions.CompanyNameEmptyException;
import fr.natan.companymicroservice.exceptions.CompanyNotFoundException;
import fr.natan.companymicroservice.model.entity.Company;
import fr.natan.companymicroservice.service.service.CompanyService;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
public class CompanyController {
    private final CompanyService companyService;

    public CompanyController(CompanyService companyService) {
        this.companyService = companyService;
    }

    @RequestMapping(value = "/companies", method = RequestMethod.POST)
    private Company createCompany(@RequestBody Company company) throws CompanyNameAlreadyExistsException,
            CompanyNameEmptyException {
        return companyService.createCompany(company);
    }

    @RequestMapping(value = "/companies", method = RequestMethod.GET)
    private List<Company> getAllCompanies(){
        return companyService.getAllCompanies();
    }

    @RequestMapping(value = "/companies/{companyID}", method = RequestMethod.GET)
    private Company getCompany(@PathVariable(name = "companyID") Long companyID) throws CompanyNotFoundException {
        return companyService.getCompanyByID(companyID);
    }

    @RequestMapping(value = "/companies/{companyID}", method = RequestMethod.PUT)
    private Company updateCompany(@PathVariable(name = "companyID") Long companyID, @RequestBody Company company)
            throws CompanyNotFoundException {
        return companyService.updateCompany(companyID,company);
    }

    @RequestMapping(value = "/companies/{companyID}", method = RequestMethod.DELETE)
    private void deleteCompany(@PathVariable(name = "companyID") Long companyID) throws CompanyNotFoundException {
        companyService.deleteCompany(companyID);
    }
}
