package fr.natan.employeemicroservice.tt_controllers;

import fr.natan.employeemicroservice.entity.Employee;
import fr.natan.employeemicroservice.exceptions.AddressNotFoundException;
import fr.natan.employeemicroservice.exceptions.EmployeeAlreadyExistsException;
import fr.natan.employeemicroservice.exceptions.EmployeeNotFoundException;
import fr.natan.employeemicroservice.exceptions.EmployeeOneOrMoreFieldsEmptyException;
import fr.natan.employeemicroservice.service.services.EmployeeService;
import fr.natan.employeemicroservice.t_openFeign.openFeign.AddressServiceOpenFeign;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
public class EmployeeController {

    private final EmployeeService employeeService;
    private final AddressServiceOpenFeign addressServiceOpenFeign;

    public EmployeeController(EmployeeService employeeService, AddressServiceOpenFeign addressServiceOpenFeign) {
        this.employeeService = employeeService;
        this.addressServiceOpenFeign = addressServiceOpenFeign;
    }

    @RequestMapping(value = "/employees/addresses/{addressID}", method = RequestMethod.POST)
    private Employee createEmployee(@RequestBody Employee employee, @PathVariable(name = "addressID") Long addressID)
            throws AddressNotFoundException, EmployeeAlreadyExistsException, EmployeeOneOrMoreFieldsEmptyException {
        return employeeService.createEmployee(employee,addressID);
    }

    @RequestMapping(value="/employees/{id}", method = RequestMethod.GET)
    private Employee getEmployee(@PathVariable(name = "id") Long employeeID) throws EmployeeNotFoundException {

        Employee employee = employeeService.getEmployeeByID(employeeID);
        employee.setAddress(addressServiceOpenFeign.findAddressById(employee.getAddressID()));
        return employee;
    }

    @RequestMapping(value = "/employees", method = RequestMethod.GET)
    private List<Employee> getAllEmployees(){
        List<Employee> employeeList = employeeService.getAllEmployees();
        employeeList.forEach(employee -> employee.setAddress(
                addressServiceOpenFeign.findAddressById(employee.getAddressID()
                )
        ));

        return employeeList;
    }

    @RequestMapping(value = "/employees/addresses/{addressID}")
    List<Employee> getEmployeesLivingAtAddress(@PathVariable(name = "addressID") Long addressID) throws AddressNotFoundException {
        List<Employee>employeeList = employeeService.getEmployeesLivingAtAddressID(addressID);
        employeeList.forEach(employee -> employee.setAddress(
                addressServiceOpenFeign.findAddressById(employee.getAddressID())
            )
        );

        return employeeList;
    }

    @RequestMapping(value = "/employees/{employeeID}", method = RequestMethod.PUT)
    private Employee updateEmployee(@PathVariable(name = "employeeID") Long employeeID, @RequestBody Employee employee)
            throws EmployeeNotFoundException, AddressNotFoundException {
        return employeeService.updateEmployee(employeeID, employee);
    }

    @RequestMapping(value = "/employees/{employeeID}", method = RequestMethod.DELETE)
    private void deleteEmployee(@PathVariable(name = "employeeID") Long employeeID) throws EmployeeNotFoundException {
        employeeService.deleteEmployee(employeeID);
    }
}
