package fr.natan.projectmicroservice.tt_controllers;

import fr.natan.projectmicroservice.exceptions.*;
import fr.natan.projectmicroservice.model.entity.Project;
import fr.natan.projectmicroservice.service.service.ProjectService;
import fr.natan.projectmicroservice.t_openFeign.openFeign.CompanyServiceOpenFeign;
import fr.natan.projectmicroservice.t_openFeign.openFeign.EmployeeServiceOpenFeign;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
public class ProjectController {
    private final ProjectService projectService;
    private final EmployeeServiceOpenFeign employeeServiceOpenFeign;
    private final CompanyServiceOpenFeign companyServiceOpenFeign;

    public ProjectController(ProjectService projectService, EmployeeServiceOpenFeign employeeServiceOpenFeign,
                             CompanyServiceOpenFeign companyServiceOpenFeign) {
        this.projectService = projectService;
        this.employeeServiceOpenFeign = employeeServiceOpenFeign;
        this.companyServiceOpenFeign = companyServiceOpenFeign;
    }

    @RequestMapping(value = "/projects/employees/{employeeID}/companies/{companyID}", method = RequestMethod.POST)
    private Project createProject(
            @RequestBody Project project,
            @PathVariable(name = "employeeID") Long employeeID,
            @PathVariable(name = "companyID") Long companyID
    ) throws ProjectAlreadyExistsException, CompanyNotFoundException,
            ProjectOneOrMoreFieldsInvalidException, EmployeeNotFoundException, ProjectDescriptionShortException {

        return projectService.createProject(project, employeeID, companyID);
    }

    @RequestMapping(value = "/projects", method = RequestMethod.GET)
    private List<Project> getAllProjects(){
        List<Project> projectList = projectService.getAllProjects();
        projectList.forEach(project -> {
            project.setCompany(companyServiceOpenFeign.getCompanyByID(project.getCompanyID())
            );
            project.setEmployee(employeeServiceOpenFeign.getEmployeeByID(project.getEmployeeID())
            );
        });
        return projectList;
    }

    @RequestMapping(value = "/projects/{id}")
    private Project getProjectByID(@PathVariable(name = "id") Long projectID) throws ProjectNotFoundException {
        Project project = projectService.getProjectByID(projectID);
        project.setEmployee(
                employeeServiceOpenFeign.getEmployeeByID(project.getEmployeeID()
                )
        );
        project.setCompany(
                companyServiceOpenFeign.getCompanyByID(project.getCompanyID()
                )
        );

        return project;
    }
    @RequestMapping(value = "/projects/{projectID}", method = RequestMethod.DELETE)
    private void deleteProject(@PathVariable(name = "projectID") Long projectID) throws ProjectNotFoundException {
        projectService.deleteProject(projectID);
    }

    @RequestMapping(value = "/projects/{projectID}",method = RequestMethod.PUT)
    private Project updateProject(@PathVariable(name = "projectID") Long projectID, @RequestBody Project project)
            throws ProjectNotFoundException, CompanyNotFoundException, EmployeeNotFoundException, ProjectOneOrMoreFieldsInvalidException,
            ProjectDescriptionShortException {

        return projectService.updateProject(projectID, project);
    }
    @RequestMapping(value = "/projects/employees/{employeeID}", method = RequestMethod.GET)
    private List<Project> getProjectsByEmployee(@PathVariable(name = "employeeID") Long employeeID) throws EmployeeNotFoundException {
        List<Project> projectList = projectService.getProjectsAffectedToEmployee(employeeID);
       projectList.forEach(project -> {
           project.setEmployee(
                   employeeServiceOpenFeign.getEmployeeByID(project.getEmployeeID())
           );

           project.setCompany(
                   companyServiceOpenFeign.getCompanyByID(project.getCompanyID())
           );
       });
        return projectList;
    }

    @RequestMapping(value = "/projects/companies/{companyID}", method = RequestMethod.GET)
    private List<Project> getProjectsByCompany(@PathVariable(name = "companyID") Long companyID) throws CompanyNotFoundException {
        List<Project> projectList = projectService.getProjectsAffectedToCompany(companyID);
        projectList.forEach(project -> {
            project.setEmployee(
                    employeeServiceOpenFeign.getEmployeeByID(project.getEmployeeID())
            );

            project.setCompany(
                    companyServiceOpenFeign.getCompanyByID(project.getCompanyID())
            );
        });

        return projectList;
    }

    @RequestMapping(value = "/projects/employees/{employeeID}/companies/{companyID}", method = RequestMethod.GET)
    private List<Project> getProjectsByEmployeeAndCompany(@PathVariable(name = "employeeID") Long employeeID,
                                                          @PathVariable(name = "companyID") Long companyID)
            throws CompanyNotFoundException, EmployeeNotFoundException {

        List<Project> projectList = projectService.getProjectsAffectedToAEmployeeInACompany(
                employeeID, companyID
        );

        projectList.forEach(project -> {
            project.setEmployee(employeeServiceOpenFeign.getEmployeeByID(project.getEmployeeID())
            );
            project.setCompany(
                    companyServiceOpenFeign.getCompanyByID(project.getCompanyID())
            );
        });

        return projectList;
    }
}
